"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const Connection_1 = __importDefault(require("../Connection"));
const schema_1 = __importDefault(require("./schema"));
exports.default = async (app) => {
    app.get('/', (request, reply) => {
        reply.send(connections.list.map(connection => connection.toObject()));
    });
    app.post('/', {
        schema: {
            body: schema_1.default.connection
        }
    }, async (request, reply) => {
        if (connections.get(request.body.name)) {
            return reply.code(409).send(new Error('Connection already exists'));
        }
        const connection = await new Connection_1.default(request.body.name).save();
        if (request.body.endpoints && request.body.endpoints.length) {
            for (let endpoint of request.body.endpoints) {
                await connection.addEndpoint(endpoint);
            }
        }
        reply.send(connection.toObject());
    });
    app.get('/:name', async (request, reply) => {
        const { name } = request.params;
        const connection = connections.get(name);
        if (!connection)
            return reply.code(404).send(new Error(`Connection ${name} not found!`));
        reply.send(connection.toObject());
    });
    app.delete('/:name', async (request, reply) => {
        const { name } = request.params;
        const connection = connections.get(name);
        if (!connection)
            return reply.code(404).send(new Error(`Connection ${name} not found!`));
        await connection.delete();
        return reply.send(connection.toObject());
    });
    app.get('/:name/endpoints', async (request, reply) => {
        const { name } = request.params;
        const connection = connections.get(name);
        if (!connection)
            return reply.code(404).send(new Error(`Connection ${name} not found!`));
        reply.send(connection.toObject().endpoints);
    });
    app.post('/:name/endpoints', {
        schema: {
            body: schema_1.default.endpoint
        }
    }, async (request, reply) => {
        const { name } = request.params;
        const connection = connections.get(name);
        if (!connection)
            return reply.code(404).send(new Error(`Connection ${name} not found!`));
        await connection.addEndpoint(request.body);
        reply.send(connection.toObject().endpoints);
    });
    app.delete('/:name/endpoints/:id', async (request, reply) => {
        const { name, id } = request.params;
        const connection = connections.get(name);
        if (!connection)
            return reply.code(404).send(new Error(`Connection ${name} not found!`));
        if (!connection.has(id))
            return reply.code(404).send(new Error(`Connection ${name} doesn't have an endpoint ${id}!`));
        await connection.removeEndpoint(id);
        reply.send(connection.toObject().endpoints);
    });
};
