"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fastify_1 = __importDefault(require("fastify"));
const basic_auth_1 = __importDefault(require("basic-auth"));
const connections_1 = __importDefault(require("./connections"));
const config_1 = __importDefault(require("./config"));
const control_1 = __importDefault(require("./control"));
const log = logger.withScope('api');
exports.default = () => {
    const app = fastify_1.default();
    if (config.api.key) {
        app.addHook('preHandler', (request, reply, next) => {
            const { authorization } = request.headers;
            if (!authorization)
                return reply.code(403).send(new Error('API key missing'));
            if (!authorization.startsWith('Bearer '))
                return reply.code(403).send(new Error('API key incorrect'));
            const key = authorization.split(' ')[1];
            if (key !== config.api.key)
                return reply.code(403).send(new Error('API key incorrect'));
            next();
        });
    }
    else {
        app.addHook('preHandler', (request, reply, next) => {
            if (!request.headers.authorization) {
                reply.code(401).header('WWW-Authenticate', 'Basic realm="Miscord API"').send();
                return;
            }
            const { name, pass } = basic_auth_1.default.parse(request.headers.authorization);
            log.debug('login', { name, pass });
            if (name === config.api.username && pass === config.api.password)
                return next();
            reply.code(403).send(new Error('Username or password incorrect.'));
        });
    }
    app.get('/', (request, reply) => {
        reply.send({ info: 'Miscord v' + require('../../package.json').version });
    });
    app.register(connections_1.default, { prefix: '/connections' });
    app.register(config_1.default, { prefix: '/config' });
    app.register(control_1.default, { prefix: '/control' });
    app.listen(config.api.port || 8000, '0.0.0.0').then(() => log.success(`API is listening on port ${config.api.port || 8000}`));
};
