"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const chalk_1 = __importDefault(require("chalk"));
const fillMissingArgs = (el) => ({ ...el, args: el.args || '' });
const buildRow = ({ name, short, args, desc }) => `${chalk_1.default.yellow('--' + name)} [${chalk_1.default.magenta('-' + short)}] ${args}  ${desc}`;
const equalPad = (param) => (line, index, arr) => ({
    ...line,
    [param]: line[param].padEnd(Math.max.apply(null, arr.map(el => el[param].length)))
});
const a = [
    {
        name: 'help',
        short: 'h',
        desc: 'shows this message'
    },
    {
        name: 'version',
        short: 'v',
        desc: 'shows version'
    },
    { name: 'dataPath',
        short: 'd',
        args: '<path>',
        desc: 'reads data from another folder'
    },
    { name: 'getPath',
        short: 'g',
        desc: 'shows default data folder path'
    },
    {
        name: 'runningWithSudoIsDangerous',
        short: 'runningWithSudoIsDangerous',
        desc: '',
        hidden: true
    }
];
function getArgs() {
    const args = require('minimist')(process.argv.slice(2));
    let newArgs = {};
    a.forEach(arg => {
        if (args[arg.name] != null)
            newArgs[arg.name] = args[arg.name];
        if (args[arg.short] != null)
            newArgs[arg.name] = args[arg.short];
        if (newArgs[arg.name] != null && arg.args && typeof newArgs[arg.name] !== 'string')
            delete newArgs[arg.name];
    });
    return newArgs;
}
exports.getArgs = getArgs;
function getHelp() {
    return a
        .filter(a => !a.hidden)
        .map(equalPad('name'))
        .map(fillMissingArgs)
        .map(equalPad('args'))
        .map(buildRow)
        .join('\n  ');
}
exports.getHelp = getHelp;
