"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const child_process_1 = require("child_process");
const gh_release_1 = __importDefault(require("gh-release"));
const path_1 = __importDefault(require("path"));
const archiver_1 = __importDefault(require("archiver"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const buildMacApp_1 = __importDefault(require("./buildMacApp"));
const { version } = require('../../package.json');
async function main() {
    console.log(`Building miscord-${version}-win.exe...`);
    await build('win-x64', 'win.exe');
    console.log(`Building miscord-${version}-win32.exe...`);
    await build('win-x86', 'win32.exe');
    console.log(`Building miscord-${version}-linux.bin...`);
    await build('linux-x64', 'linux.bin');
    console.log(`Building miscord-${version}-linux32.bin...`);
    await build('linux-x86', 'linux32.bin');
    console.log(`Building miscord-${version}-mac.bin...`);
    await build('mac-x64', 'mac.bin');
    console.log(`Building miscord-${version}-mac.zip...`);
    await buildMacApp_1.default(version);
    gh_release_1.default({
        auth: {
            token: process.env.GITHUB_TOKEN
        },
        owner: 'miscord',
        repo: 'miscord',
        tag_name: 'v' + version,
        target_commitish: 'master',
        name: version,
        assets: [
            'win',
            'win32',
            'linux',
            'linux32',
            'mac',
            'macapp'
        ].map(platform => `build/miscord-${version}-${platform}.zip`)
    }, (err, result) => {
        if (err)
            console.error(err);
        console.log(result);
        process.exit(0);
    });
}
async function build(nodeVersion, name) {
    const filename = `miscord-${version}-${name}`;
    await exec(`npx pkg -t latest-${nodeVersion} --public . -o ./build/${filename}`);
    await archivize(filename);
    await fs_extra_1.default.remove(path_1.default.join('build', filename));
}
function exec(command) {
    return new Promise((resolve, reject) => {
        const child = child_process_1.spawn('/bin/bash', ['-c', command], { cwd: require('path').join(__dirname, '..'), env: process.env });
        child.stdout.on('data', data => process.stdout.write(data));
        child.stderr.on('data', data => process.stderr.write(data));
        child.on('error.ts', err => {
            console.error(err);
            process.exit(1);
        });
        child.on('close', () => resolve());
    });
}
const archivize = (file) => new Promise((resolve, reject) => {
    const { name, base } = path_1.default.parse(file);
    const stream = fs_extra_1.default.createWriteStream(path_1.default.join('build', name + '.zip'));
    stream.on('close', resolve);
    const archive = archiver_1.default('zip', {
        zlib: { level: 9 } // Sets the compression level.
    });
    archive.pipe(stream);
    archive.append(fs_extra_1.default.createReadStream(path_1.default.join('build', file)), { name: base, mode: 0o755 });
    archive.finalize();
});
main();
