"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_extra_1 = __importDefault(require("fs-extra"));
const path_1 = __importDefault(require("path"));
const node_fetch_1 = __importDefault(require("node-fetch"));
const archiver_1 = __importDefault(require("archiver"));
const extract_zip_1 = __importDefault(require("extract-zip"));
const tmp_1 = __importDefault(require("tmp"));
async function buildMacApp(version) {
    // create a temporary directory
    const tmpdir = tmp_1.default.dirSync();
    const tmppath = (p) => path_1.default.join(tmpdir.name, p);
    // get path to the zip file which will be downloaded
    const zipPath = tmppath('miscord.zip');
    // download the template app from GitHub
    await download('https://github.com/miscord/macos-wrapper/releases/download/v1.0.1/Miscord.zip', zipPath);
    // extract it to the temporary directory
    await extract(zipPath, { dir: tmpdir.name });
    // delete the zip file
    await fs_extra_1.default.remove(zipPath);
    // copy the zipped binary
    await fs_extra_1.default.copyFile(`build/miscord-${version}-mac.zip`, tmppath('miscord-mac.zip'));
    // extract the binary
    await extract(tmppath(`miscord-mac.zip`), { dir: tmpdir.name });
    // fill the template with a binary
    await fs_extra_1.default.copyFile(tmppath(`miscord-${version}-mac.bin`), tmppath('Miscord.app/Contents/Resources/miscord-mac.bin'));
    // create a zip file
    await archivize(zipPath, tmppath('Miscord.app'));
    // copy the zip file to the build folder
    await fs_extra_1.default.copyFile(zipPath, `build/miscord-${version}-macapp.zip`);
    // delete the temporary directory
    await fs_extra_1.default.remove(tmpdir.name);
}
exports.default = buildMacApp;
const extract = (path, opts) => new Promise((resolve, reject) => extract_zip_1.default(path, opts, err => { if (err)
    reject(err);
else
    resolve(); }));
const download = (url, path) => new Promise(async (resolve, reject) => {
    const stream = fs_extra_1.default.createWriteStream(path);
    const res = await node_fetch_1.default(url);
    res.body.pipe(stream);
    stream.on('close', resolve);
});
const archivize = (filePath, directory) => new Promise((resolve, reject) => {
    const stream = fs_extra_1.default.createWriteStream(filePath);
    stream.on('close', resolve);
    const archive = archiver_1.default('zip', {
        zlib: { level: 9 } // Sets the compression level.
    });
    archive.pipe(stream);
    archive.directory(directory, path_1.default.parse(directory).base);
    archive.finalize();
});
