#!/usr/bin/env node
"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const chalk_1 = __importDefault(require("chalk"));
const sudo_block_1 = __importDefault(require("sudo-block"));
const cluster_1 = __importDefault(require("cluster"));
const Logger_1 = __importDefault(require("../logger/Logger"));
// @ts-ignore
global.logger = new Logger_1.default(process.env.MISCORD_LOG_LEVEL || 'info');
const FileConfig_1 = require("../config/FileConfig");
const worker_1 = __importDefault(require("./worker"));
const arguments_1 = require("../arguments");
const help_1 = __importDefault(require("./help"));
const { version } = require('../../package.json');
const fork = (d) => cluster_1.default.fork({ DATA_PATH: d, ...process.env }).on('online', () => { lastRunTime = new Date(); });
let lastRunTime;
if (cluster_1.default.isMaster) {
    const printAndExit = (m) => { console.log(m); process.exit(0); };
    const outdated = 'Hey! Your version of Node.JS seems outdated. Minimum version required: v8.5.0, your version: ' + process.version;
    if (!require('semver').gte(process.version, '8.5.0'))
        printAndExit(chalk_1.default.yellow(outdated));
    const args = arguments_1.getArgs();
    if (args.help)
        printAndExit(help_1.default);
    if (args.version)
        printAndExit(version);
    if (args.getPath)
        printAndExit(FileConfig_1.getConfigDir());
    const defaultMessage = chalk_1.default `
{red.bold Miscord should not be run with root permissions.}
If running without {bold sudo} doesn't work, you can either fix your permission problems or change where npm stores global packages by putting {bold ~/npm/bin} in your PATH and running:
{blue npm config set prefix ~/npm}
See: {underline https://github.com/sindresorhus/guides/blob/master/npm-global-without-sudo.md}
If you {underline really} need to run Miscord with {bold sudo}, add parameter {bold --runningWithSudoIsDangerous}.
`;
    if (!args.runningWithSudoIsDangerous)
        sudo_block_1.default(defaultMessage);
    fork(args.dataPath);
    let loginFailed = true;
    cluster_1.default.on('message', (worker, message) => {
        if (message === 'login successful')
            loginFailed = false;
    });
    cluster_1.default.on('exit', (worker, code, signal) => {
        logger.error(`Worker process ${worker.process.pid} died (${code}, ${signal}).`);
        // Code 0: exit
        // Code 1: error
        // Code 2: restart
        if ((Date.now() - lastRunTime.getTime()) < (2 * 1000)) {
            logger.fatal('Process crashed less than 2 seconds since the last launch, exiting.');
            process.exit(1);
        }
        if (code === 0) {
            logger.success('Quit signal received, exiting...');
            process.exit(0);
        }
        if (loginFailed && code === 1) {
            logger.fatal('Logging in failed, exiting.');
            process.exit(1);
        }
        loginFailed = true;
        fork(args.dataPath);
    });
}
else {
    worker_1.default();
}
