"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
class Command {
    constructor(handler, options) {
        this.handler = handler;
        if (options)
            return new ExtendedCommand(handler, options);
    }
    async run(argv, channel) {
        let reply = await this.handler(argv, (message, params) => channel.send(message, params));
        if (reply)
            channel.send(reply);
    }
}
exports.default = Command;
class ExtendedCommand extends Command {
    constructor(handler, options) {
        super(handler);
        this.skipCheck = false;
        this.argc = 0;
        this.usage = '';
        this.example = '';
        this.allowMoreArguments = false;
        this.argc = options.argc;
        this.usage = options.usage;
        this.example = options.example;
        this.allowMoreArguments = options.allowMoreArguments || false;
    }
    async run(argv, channel) {
        if (!this.skipCheck && this.handleArgc(argv, channel))
            return;
        return super.run(argv, channel);
    }
    handleArgc(argv, channel) {
        if (argv.length === this.argc)
            return false;
        if (argv.length > this.argc && this.allowMoreArguments)
            return false;
        channel.send(`Too ${argv.length < this.argc ? 'few' : 'many'} arguments!
  Usage: ${this.usage}
  Example: ${this.example}`);
        return true;
    }
}
exports.ExtendedCommand = ExtendedCommand;
