"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const dot_prop_1 = __importDefault(require("dot-prop"));
const defaultConfig_1 = __importDefault(require("./defaultConfig"));
const mergeDeep_1 = __importDefault(require("./mergeDeep"));
class Config {
    constructor() {
        this.config = defaultConfig_1.default;
    }
    static proxify(config) {
        return new Proxy(config, {
            get: (target, name, receiver) => Reflect.get(target, name, receiver) || Reflect.get(target.config, name, receiver)
        });
    }
    _load(config) {
        this.config = mergeDeep_1.default(this.config, config);
        logger.setLevel(process.env.MISCORD_LOG_LEVEL || this.config.logLevel || 'info');
    }
    _set(key, value) {
        if (value != null) {
            dot_prop_1.default.set(this.config, key, value);
        }
        else {
            dot_prop_1.default.delete(this.config, key);
        }
    }
    validate() {
        if (!this.config.discord.token || !this.config.messenger.username || !this.config.messenger.password) {
            if (process.pkg && process.platform === 'win32') {
                logger.fatal('Token/username/password not found.');
                process.stdin.resume();
                return;
            }
            throw new Error('Token/username/password not found.');
        }
    }
}
exports.default = Config;
