"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
// @ts-ignore
const fromEntries = (entries) => Object.assign(...entries.map(([k, v]) => ({ [k]: v })));
const os_1 = __importDefault(require("os"));
const path_1 = __importDefault(require("path"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const is_docker_1 = __importDefault(require("is-docker"));
const js_yaml_1 = __importDefault(require("js-yaml"));
const Config_1 = __importDefault(require("./Config"));
const diff_1 = __importDefault(require("./diff"));
const defaultConfig_1 = __importDefault(require("./defaultConfig"));
const log = logger.withScope('FileConfig');
class FileConfig extends Config_1.default {
    constructor(dataPath = getConfigDir()) {
        super();
        dataPath = path_1.default.resolve(dataPath);
        this.path = path_1.default.join(dataPath, 'config.json');
        this.connPath = path_1.default.join(dataPath, 'connections.yml');
        this.sessionPath = path_1.default.join(dataPath, 'session.json');
    }
    async load() {
        if (!fs_extra_1.default.existsSync(this.path)) {
            log.warn(`${this.path} not found, creating example config`);
            const example = path_1.default.join(__dirname, '../../config.example.json');
            await fs_extra_1.default.ensureDir(path_1.default.parse(this.path).dir);
            // https://github.com/zeit/pkg/issues/342#issuecomment-368303496
            if (process.pkg) {
                fs_extra_1.default.writeFileSync(this.path, fs_extra_1.default.readFileSync(example));
            }
            else {
                fs_extra_1.default.copyFileSync(example, this.path);
            }
            log.fatal(`Default config copied to ${this.path}
Fill it with data or use config generator here:
https://miscord.net/config-generator.html`);
        }
        log.info(`Loading config from ${this.path}`);
        const config = await fs_extra_1.default.readJSON(this.path);
        this._load(config);
    }
    set(key, value) {
        this._set(key, value);
        return this.save();
    }
    async save() {
        let config = JSON.parse(JSON.stringify(this.config));
        config = fromEntries(Object.entries(config)
            .map(([key, value]) => {
            if (typeof value === 'object' && defaultConfig_1.default[key]) {
                return [key, diff_1.default(defaultConfig_1.default[key], value)];
            }
            return [key, value];
        }));
        await fs_extra_1.default.writeJSON(this.path, config, { spaces: 2 });
    }
    async loadConnections() {
        const file = await fs_extra_1.default.readFile(this.connPath, 'utf8');
        return js_yaml_1.default.safeLoad(file);
    }
    saveConnections(connections) {
        return fs_extra_1.default.writeFile(this.connPath, js_yaml_1.default.safeDump(connections), 'utf8');
    }
    loadSession() { return fs_extra_1.default.readJSON(this.sessionPath); }
    saveSession(session) { return fs_extra_1.default.writeJSON(this.sessionPath, session); }
}
exports.default = FileConfig;
function getConfigDir() {
    if (is_docker_1.default())
        return '/config';
    switch (process.platform) {
        case 'win32':
            return path_1.default.join(process.env.APPDATA, 'Miscord');
        case 'linux':
            return path_1.default.join(os_1.default.homedir(), '.config', 'Miscord');
        case 'darwin':
            return path_1.default.join(os_1.default.homedir(), 'Library', 'Application Support', 'Miscord');
        default:
            return path_1.default.join(os_1.default.homedir(), '.miscord');
    }
}
exports.getConfigDir = getConfigDir;
