"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const Config_1 = __importDefault(require("./Config"));
const node_fetch_1 = __importDefault(require("node-fetch"));
function getURL(filename) {
    let env = process.env.STORAGE_URL;
    return env + (env.endsWith('/') ? filename : '/' + filename);
}
async function getJSON(url) {
    const res = await node_fetch_1.default(url);
    return res.json();
}
function postJSON(url, body) {
    return node_fetch_1.default(url, {
        method: 'POST',
        headers: {
            'Content-Type': 'application/json'
        },
        body: JSON.stringify(body)
    });
}
class RemoteConfig extends Config_1.default {
    constructor() {
        super(...arguments);
        this.url = getURL('config.json');
        this.connURL = getURL('connections.json');
        this.sessionURL = getURL('session.json');
    }
    async load() {
        const config = await getJSON(this.url);
        this._load(config);
    }
    set(key, value) {
        this._set(key, value);
        return postJSON(this.url, { [key]: value });
    }
    loadConnections() { return getJSON(this.connURL); }
    async saveConnections(connections) { postJSON(this.connURL, connections); }
    loadSession() { return getJSON(this.sessionURL); }
    async saveSession(session) { postJSON(this.sessionURL, session); }
}
exports.default = RemoteConfig;
