"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const log = logger.withScope('downloadFile');
const fs_extra_1 = __importDefault(require("fs-extra"));
const url_1 = __importDefault(require("url"));
const path_1 = __importDefault(require("path"));
const tmp_1 = __importDefault(require("tmp"));
const http_1 = __importDefault(require("http"));
const https_1 = __importDefault(require("https"));
exports.default = async (attachURL) => {
    log.debug('Downloading a file from URL:', attachURL);
    const tempdir = tmp_1.default.dirSync().name;
    await fs_extra_1.default.ensureDir(tempdir);
    const filePath = await download(tempdir, attachURL);
    log.debug('Downloaded file path:', filePath);
    const extension = path_1.default.parse(filePath).ext;
    log.debug('File extension:', extension);
    return { filePath, extension };
};
async function download(dir, attachURL) {
    let filePath = path_1.default.join(dir, url_1.default.parse(attachURL).pathname.split('/').slice(-1).pop());
    log.debug('Download target:', filePath);
    while (fs_extra_1.default.existsSync(filePath)) {
        const parsed = path_1.default.parse(filePath);
        if (/^.*\d$/.test(parsed.name)) {
            let [, text, digits] = Array.from(parsed.name.match(/^(.*?)(\d+)$/));
            parsed.name = text + (Number(digits) + 1);
        }
        else {
            parsed.name = parsed.name + '1';
        }
        filePath = path_1.default.join(parsed.dir, parsed.name + parsed.ext);
        log.debug('Target already exists, new path:', filePath);
    }
    const fileStream = fs_extra_1.default.createWriteStream(filePath);
    const stream = await new Promise((resolve, reject) => {
        const cb = (res) => resolve(res);
        if (url_1.default.parse(attachURL).protocol === 'https:') {
            https_1.default.get(attachURL, cb);
        }
        else {
            http_1.default.get(attachURL, cb);
        }
    });
    await new Promise((resolve, reject) => stream.pipe(fileStream).on('finish', resolve));
    return path_1.default.resolve(filePath);
}
