"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const log = logger.withScope('createMessage:fromDiscord:messenger');
const downloadFile_1 = __importDefault(require("../downloadFile"));
const handleCustomEmoji_1 = __importDefault(require("../handleCustomEmoji"));
exports.default = async (message) => {
    let username = message.member ? (message.member.nickname || message.author.username) : message.author.username;
    let content = message.cleanContent;
    content = handleCustomEmoji_1.default(content);
    // copy message content to a new variable, as the cleanContent property is read-only
    log.debug('clean content', content);
    let embedImageURL;
    // parse embed into plaintext
    if (message.embeds.length > 0 && !config.messenger.ignoreEmbeds) {
        message.embeds.forEach(embed => {
            if (embed.title)
                content += '\n' + embed.title;
            if (embed.url && !content.includes(embed.url))
                content += '\n(' + embed.url + ')';
            if (embed.description)
                content += '\n' + embed.description;
            embed.fields.forEach(field => { content += '\n\n' + field.name + '\n' + field.value; });
        });
        log.debug('content with embed', content);
        // get image url from discord embeds
        embedImageURL = message.embeds.length > 0
            ? (message.embeds[0].image
                ? message.embeds[0].image.url
                : (message.embeds[0].thumbnail
                    ? message.embeds[0].thumbnail.url
                    : undefined))
            : undefined;
    }
    const body = config.messenger.format
        .replace('{username}', username)
        .replace('{content}', content)
        .replace('{message}', content)
        .replace('{source}', config.messenger.sourceFormat.discord)
        .replace('{newline}', '\n');
    const attachmentURLs = [config.messenger.ignoreEmbeds ? null : embedImageURL]
        .concat(message.attachments.map(attach => attach.url))
        .filter(Boolean);
    const attachments = await Promise.all(attachmentURLs.map(downloadFile_1.default));
    return { body, attachments };
};
