"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const getAttachmentURL_1 = __importDefault(require("../getAttachmentURL"));
const log = logger.withScope('createMessage:fromMessenger:discord');
const handleEmojis_1 = __importDefault(require("../../discord/handleEmojis"));
const handleMentions_1 = __importDefault(require("../../discord/handleMentions"));
const emojiCount_1 = __importDefault(require("../emojiCount"));
const parseMessengerMessage_1 = __importDefault(require("./parseMessengerMessage"));
const url_1 = __importDefault(require("url"));
const thumbs = [369239263222822, 369239383222810, 369239343222814];
exports.default = async (thread, sender, message) => {
    // set description to message body, set author to message sender
    let authorName;
    ({ authorName, message } = parseMessengerMessage_1.default(thread, sender, message));
    let body = handleEmojis_1.default(handleMentions_1.default(message.message));
    if (body.length > 2000)
        body = body.slice(0, 1997) + '...';
    // if there are no attachments, send it already
    const opts = {
        username: authorName.length <= 32 ? emojiCount_1.default(authorName) === 1 ? authorName + '.' : authorName : authorName.substr(0, 29) + '...',
        avatarURL: sender.profilePicLarge,
        files: []
    };
    if ((!message.attachments || !message.attachments.length) && !message.stickerId)
        return { body, opts };
    const files = message.attachments.filter(attach => attach.type.endsWith('Attachment'));
    const xma = message.attachments.filter(attach => attach.type.endsWith('XMA'));
    log.trace('attachments to parse', message.attachments);
    for (let attach of files) {
        const url = attach.url || await getAttachmentURL_1.default(message, attach);
        if (!url)
            continue;
        if (attach.size && attach.size > 8 * 1024 * 1024) {
            log.warn('Attachment was not sent due to Discord file size limit', attach);
        }
        opts.files.push({ attachment: url, name: attach.filename });
    }
    const appendToBody = (str) => { if (!body.includes(str))
        body += '\n' + str; };
    for (let attach of xma) {
        if (attach.message)
            appendToBody(attach.message);
        if (attach.description)
            appendToBody(attach.description);
        if (attach.url)
            appendToBody(attach.url);
        if (attach.imageURL) {
            opts.files.push({
                attachment: attach.imageURL,
                name: url_1.default.parse(attach.imageURL).pathname.split('/').slice(-1)[0]
            });
        }
    }
    if (message.stickerId && !thumbs.includes(message.stickerId)) {
        const stickerURL = await messenger.client.getStickerURL(message.stickerId);
        opts.files.push({ attachment: stickerURL, name: message.stickerId + '.png' });
    }
    log.trace('files', opts.files);
    return { body, opts };
};
