"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const getAttachmentURL_1 = __importDefault(require("../getAttachmentURL"));
const log = logger.withScope('createMessage:fromMessenger:messenger');
const downloadFile_1 = __importDefault(require("../downloadFile"));
const parseMessengerMessage_1 = __importDefault(require("./parseMessengerMessage"));
const thumbs = [369239263222822, 369239383222810, 369239343222814];
exports.default = async (thread, sender, message, source) => {
    // set description to message body, set author to message sender
    let authorName;
    ({ authorName, message } = parseMessengerMessage_1.default(thread, sender, message));
    let body = config.messenger.format
        .replace('{username}', authorName)
        .replace('{content}', message.message)
        .replace('{message}', message.message)
        .replace('{source}', config.messenger.sourceFormat.messenger.replace('{name}', source))
        .replace('{newline}', '\n');
    if ((!message.attachments || !message.attachments.length) && !message.stickerId)
        return { body };
    const files = message.attachments.filter(attach => attach.type.endsWith('Attachment'));
    const xma = message.attachments.filter(attach => attach.type.endsWith('XMA'));
    const attachments = [];
    for (let attach of files) {
        const url = attach.url ? attach.url : await getAttachmentURL_1.default(message, attach);
        if (!url)
            continue;
        attachments.push(await downloadFile_1.default(url));
    }
    const appendToBody = (str) => { if (!body.includes(str))
        body += '\n' + str; };
    for (let attach of xma) {
        if (attach.message)
            appendToBody(attach.message);
        if (attach.description)
            appendToBody(attach.description);
        if (attach.url)
            appendToBody(attach.url);
        if (attach.imageURL)
            attachments.push(await downloadFile_1.default(attach.imageURL));
    }
    if (message.stickerId && !thumbs.includes(message.stickerId)) {
        attachments.push(await messenger.client.getStickerURL(message.stickerId).then(downloadFile_1.default));
    }
    log.trace('attachments', attachments);
    return { body, attachments };
};
