"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const log = logger.withScope('discord:login');
const discord_js_1 = require("discord.js");
const GuildArray_1 = __importDefault(require("../types/GuildArray"));
exports.default = () => {
    const client = new discord_js_1.Client();
    // log in to discord
    log.start('Logging in to Discord...');
    return client.login(config.discord.token).then(async () => {
        log.success('Logged in to Discord');
        global.discord = {
            client,
            guilds: new GuildArray_1.default(...client.guilds.array()),
            channels: {},
            webhooks: new discord_js_1.Collection()
        };
        // set activity
        await client.user.setActivity('Miscord v' + require('../../package.json').version);
        // if bot isn't added to any guilds, send error
        if (client.guilds.size === 0) {
            throw new Error(`No guilds added!
You can add a bot to your guild here:
https://discordapp.com/api/oauth2/authorize?client_id=${client.user.id}&permissions=537390096&scope=bot
It's not an error... unless you added the bot to your guild already.`);
        }
        // set guild as a global variable, if it's specified in arguments then get it by name, if not then get the first one
        if (config.discord.guild) {
            const guild = client.guilds.find(guild => guild.name.toLowerCase() === config.discord.guild.toLowerCase() || guild.id === config.discord.guild);
            if (!guild)
                throw new Error(`Guild ${config.discord.guild} was not found!`);
            // move OUR guild to the front.
            discord.guilds = new GuildArray_1.default(guild, ...discord.guilds.filter(_guild => _guild.id !== guild.id));
        }
        log.debug('Found Discord guild(s)');
        log.trace('guilds', discord.guilds, 1);
        // if user put category name in the config
        if (config.discord.category) {
            // get category
            const category = discord.guilds[0].channels.find(channel => (channel.name.toLowerCase() === config.discord.category.toLowerCase() || channel.id === config.discord.category) &&
                (channel.type === null || channel.type === 'category'));
            // if category is missing, crash
            if (!category)
                throw new Error(`Category ${config.discord.category} was not found!`);
            log.debug('Got Discord category');
            log.trace('category', category);
            discord.category = category;
        }
        log.debug('Got Discord channels list');
        discord.channels = {};
        if (config.channels.error) {
            discord.channels.error = await getChannel(client, config.channels.error);
        }
        if (config.channels.command) {
            if (!Array.isArray(config.channels.command))
                config.channels.command = [config.channels.command];
            discord.channels.command = (await Promise.all(config.channels.command.map(id => getChannel(client, id))
            // .filter(channel => channel != null)
            )).filter(Boolean);
        }
        client.on('error', (err) => { throw err; });
        return client;
    });
};
async function getChannel(client, channelID) {
    if (discord.client.users.has(channelID))
        return client.users.get(channelID).createDM();
    if (discord.client.channels.has(channelID))
        return client.channels.get(channelID);
    log.warn(`Channel/user ${channelID} not found.`);
}
