"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = __importDefault(require("path"));
const FileConfig_1 = require("../config/FileConfig");
const strip_ansi_1 = __importDefault(require("strip-ansi"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const is_docker_1 = __importDefault(require("is-docker"));
const timezonedDate_1 = __importDefault(require("./timezonedDate"));
class FakeWriteStream {
    end(cb) { if (cb)
        cb(); }
    write(str, encoding, cb) {
        return true;
    }
}
const getWritableDate = () => timezonedDate_1.default().replace('T', '_').replace(/:/g, '-');
exports.default = (configPath = FileConfig_1.getConfigDir()) => {
    let logStream;
    if (process.env.STORAGE_URL) {
        logStream = new FakeWriteStream();
    }
    else {
        const dir = path_1.default.join(configPath, 'logs');
        fs_extra_1.default.ensureDirSync(dir);
        const filename = getWritableDate() + '.log';
        logStream = fs_extra_1.default.createWriteStream(path_1.default.join(dir, filename));
    }
    let line = '';
    // https://gist.github.com/pguillory/729616/32aa9dd5b5881f6f2719db835424a7cb96dfdfd6
    function bindWrite(write, stream) {
        return (str, encoding, fd) => {
            if (str.startsWith('close')) {
                const match = str.match(/close (\d+)/);
                if (match) {
                    const code = parseInt(match[1], 10);
                    return logStream.end(() => process.exit(code));
                }
            }
            line += str;
            if (!str.includes('\n'))
                return;
            write.call(process[stream], is_docker_1.default() ? strip_ansi_1.default(line) : line, encoding, fd);
            logStream.write(strip_ansi_1.default(line), encoding, fd);
            line = '';
        };
    }
    // @ts-ignore
    process.stdout.write = bindWrite(process.stdout.write, 'stdout');
    // @ts-ignore
    process.stderr.write = bindWrite(process.stderr.write, 'stderr');
};
