"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const getThreadName_1 = __importDefault(require("./getThreadName"));
const log = logger.withScope('messenger:getThread');
exports.default = async (threadID, useCache = true) => {
    if (messenger.threads.has(threadID) && useCache) {
        log.debug('Messenger thread cached');
        let thread = messenger.threads.get(threadID);
        if (!thread)
            throw new Error(`Could not find thread ${threadID}`);
        return thread;
    }
    else {
        log.debug('Getting Messenger thread from API');
        const thread = await messenger.client.getThreadInfo(threadID);
        if (!thread)
            throw new Error(`Could not find thread ${threadID}`);
        const miscordThread = await fillNames(thread);
        messenger.threads.set(threadID, miscordThread);
        return miscordThread;
    }
};
async function fillNames(thread) {
    thread.name = await getThreadName_1.default(thread, false);
    thread.cleanName = await getThreadName_1.default(thread);
    return thread;
}
exports.fillNames = fillNames;
