"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const log = logger.withScope('messenger:handleEvent');
const index_1 = require("./index");
const handlePlan_1 = __importDefault(require("./handlePlan"));
const handlePoll_1 = __importDefault(require("./handlePoll"));
async function handleEvent(_event) {
    log.trace('event', _event);
    const { event, type } = _event;
    if (type === 'readReceiptEvent' || type === 'deliveryReceiptEvent')
        return;
    const messageEvent = event;
    if (type.startsWith('event'))
        return handlePlan_1.default({ type, event: event });
    if (type.startsWith('poll'))
        return handlePoll_1.default({ type, event: event });
    if (!config.messenger.handleEvents)
        return;
    const thread = await index_1.getThread(event.threadId, type !== 'threadNameEvent');
    log.debug('Got Messenger thread');
    log.trace('thread', thread, 1);
    if (!thread)
        return;
    let connection = await connections.getWithCreateFallback(event.threadId, thread.cleanName);
    if (!connection)
        return;
    if (type === 'threadNameEvent')
        await connection.checkChannelRenames(thread.name);
    const channels = connection.getWritableChannels();
    channels.forEach(async ({ channel }) => {
        channel.send(`*${connection.getThreads().length > 1 ? thread.name + ': ' : ''}${messageEvent.message}*`);
    });
    const threads = connection.getOtherWritableThreads(event.threadId);
    threads.forEach(async (_thread) => {
        log.debug('Sending Messenger message');
        const info = await messenger.client.sendMessage(_thread.id, `_${thread.name}: ${messageEvent.message}_`);
        log.trace('sent message info', info);
        log.debug('Sent message on Messenger');
    });
}
exports.default = handleEvent;
