"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const log = logger.withScope('messenger:handlePoll');
const index_1 = require("./index");
const utils_1 = require("../utils");
exports.default = async (_poll) => {
    log.trace('poll', _poll);
    const { event: poll } = _poll;
    if (!config.messenger.handlePolls)
        return;
    const thread = await index_1.getThread(poll.threadId);
    log.debug('Got Messenger thread');
    log.trace('thread', thread, 1);
    let connection = await connections.getWithCreateFallback(poll.threadId.toString(), thread.cleanName);
    if (!connection)
        return;
    const options = await Promise.all(poll.options.map(async (option) => {
        return {
            ...option,
            voters: (await Promise.all(option.voters.map(voter => index_1.getSender(voter)))).map(el => el.name)
        };
    }));
    const description = config.messenger.showPollDetails
        ? `*${poll.message}*
${options.map(option => `${option.title} (${option.voteCount}): ${utils_1.truncatePeople(option.voters)}`).join('\n')}
` : poll.message;
    const channels = await connection.getWritableChannels();
    channels.forEach(async ({ channel }) => {
        channel.send({ embed: { title: `Poll: **${poll.title}**`, description } });
    });
    const threads = connection.getOtherWritableThreads(poll.threadId.toString());
    threads.forEach(async (_thread) => {
        log.debug('Sending Messenger message');
        const info = await messenger.client.sendMessage(_thread.id, `_${thread.name} - ${poll.title}_:\n${description}`);
        log.trace('sent message info', info);
        log.debug('Sent message on Messenger');
    });
};
