"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const log = logger.withScope('messenger:listener');
const fs_extra_1 = __importDefault(require("fs-extra"));
const createMessage_1 = require("../createMessage");
const discord_1 = require("../discord");
const utils_1 = require("../utils");
const _1 = require("./");
exports.default = async (message) => {
    if (!message)
        throw new Error('Message missing!');
    log.trace('message', message);
    log.info('Got a Messenger message');
    if (utils_1.checkMKeep(message.message))
        return log.debug('m!keep received, ignoring.');
    if (utils_1.checkIgnoredSequences(message.message))
        return log.debug('found an ignored sequence, ignoring.');
    // get thread info to know if it's a group conversation
    const thread = await _1.getThread(message.threadId);
    log.debug('Got Messenger thread');
    log.trace('thread', thread, 1);
    // also get sender info
    const sender = await _1.getSender(message.authorId);
    if (!sender)
        return;
    log.debug('Got user info');
    log.trace('sender', sender);
    let connection = await connections.getWithCreateFallback(message.threadId.toString(), thread.cleanName);
    if (!connection)
        return;
    await connection.checkChannelRenames(thread.cleanName);
    // get channel
    const channels = await connection.getWritableChannels();
    log.debug('Got Discord channels');
    const { body, opts } = await createMessage_1.fromMessenger.toDiscord(thread, sender, message);
    channels.map(async (endpoint) => {
        if (!body && !(opts.files && opts.files.length))
            return log.warn('Not sending message, empty.');
        return discord_1.sendMessage(endpoint.channel, body, opts, thread.image);
    });
    // check if it needs resending (linked channels)
    const threads = connection.getOtherWritableThreads(message.threadId);
    const { body: mBody, attachments } = await createMessage_1.fromMessenger.toMessenger(thread, sender, message, thread.cleanName);
    log.debug('Created Messenger message');
    Promise.all(threads.map(async (_thread) => {
        if (mBody && mBody.trim()) {
            log.debug('Sending Messenger message');
            const info = await messenger.client.sendMessage(_thread.id, mBody.toString());
            log.trace('sent message info', info);
            log.debug('Sent message on Messenger');
        }
        if (attachments && attachments.length) {
            log.debug('Sending Messenger attachments');
            const info = await Promise.all(attachments.map(attachment => messenger.client.sendAttachmentFile(_thread.id, attachment.filePath, attachment.extension)));
            log.trace('sent attachments info', info);
            log.debug('Sent Messenger attachments');
        }
    })).then(async () => {
        if (attachments && attachments.length) {
            await Promise.all(attachments.map(attachment => fs_extra_1.default.unlink(attachment.filePath)));
            log.debug('Removed temporary attachment files');
        }
    });
};
