"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const log = logger.withScope('updateNotifier');
const fs_extra_1 = __importDefault(require("fs-extra"));
const semver_1 = __importDefault(require("semver"));
const chalk_1 = __importDefault(require("chalk"));
const node_fetch_1 = __importDefault(require("node-fetch"));
exports.default = async () => {
    const pkg = await fs_extra_1.default.readJSON('../package.json');
    log.info('Checking for updates...');
    let res;
    try {
        res = await node_fetch_1.default('https://api.github.com/repos/miscord/miscord/releases/latest', {
            headers: { 'User-Agent': 'Miscord v' + pkg.version }
        });
    }
    catch (err) {
        log.error('Something went wrong', err);
        return;
    }
    const release = await res.json();
    log.trace('release', release);
    if (!release.tag_name)
        return log.warn('Couldn\'t check updates', release);
    const latest = release.tag_name.substring(1, release.tag_name.length);
    log.trace('release tag', release.tag_name);
    log.debug('release version', latest);
    log.debug('package version', pkg.version);
    if (semver_1.default.lt(pkg.version, latest))
        console.log(chalk_1.default.green(`New version ${latest} available!`), chalk_1.default.cyan(`\nChangelog:\n${release.body}`));
};
